//
// Random Program Generator
// Bryan Turner (bryan.turner@pobox.com)
// July, 2005
//

#include "Common.h"
#include "Block.h"
#include "Type.h"
#include "Function.h"
#include "Statement.h"
#include "Variable.h"

bool BlockProbability( Block &block )
{
	if ( block.stm.empty() ) return true;	// Always have at least one statement in the block.
	return (genrand() % 1000) > 500;			// 50% probability for each additional statement.  Should upgrade this to generate a uniform distribution around a selected block size (or function size?)
}

Block::Block( Function &curFunc, bool depthProtect )
{
	curFunc.STMT_DEPTH++;
	curFunc.stack.push_back( this );		// Push this block onto the variable scope stack..
	
	bDepthProtect = depthProtect;			// Signal the back-end to output a depth protection statement;
	
	while ( BlockProbability( *this ) )
	{
		stm.push_back( new Statement( curFunc ) );
	}

	curFunc.stack.pop_back();
	curFunc.STMT_DEPTH--;
}

void OutputStatement( Statement *stm, std::ostream *pOut )
{
	stm->Output( *pOut );
}

void OutputStatementList( vector<Statement*> &stm, std::ostream &out )
{
	for_each( stm.begin(), stm.end(), std::bind2nd(std::ptr_fun(OutputStatement), &out) );
}

void Block::Output( std::ostream &out )
{
	out << "{" << eol;
	
	if ( bDepthProtect )
	{
		out << "DEPTH++;" << eol;
	}
	
	OutputVariableList( local_vars, out );
	OutputStatementList( stm, out );
	
	if ( bDepthProtect )
	{
		out << "DEPTH--;" << eol;
	}
	
	out << "}" << eol;
};

