//
// Random Program Generator
// Bryan Turner (bryan.turner@pobox.com)
// July, 2005
//

#include "Common.h"
#include "FunctionInvocation.h"
#include "Expression.h"
#include "Type.h"
#include "Function.h"

static vector<bool> needcomma;					// Flag to track output of commas

FunctionInvocation::FunctionInvocation( Function &curFunc, bool bStandardFunc )
{
	isStandardFunc = bStandardFunc;
	if ( isStandardFunc )
	{
		eFunc = (eBinaryOps)(genrand() % MAX_BINARY_OP);
		
		// TODO: Generate an expression with the correct type...
		param_value.push_back( new Expression( curFunc ) );
		param_value.push_back( new Expression( curFunc ) );
	}
	else
	{
		func = SelectFunction( curFunc, isBackLink );
		for ( unsigned int i=0; i<func->param.size(); i++ )
		{
			// TODO: Generate an expression with the correct type...
			param_value.push_back( new Expression( curFunc ) );
		}
	}
}

FunctionInvocation::FunctionInvocation( Function *target )
{
	isStandardFunc = false;
	func = target;
	for ( unsigned int i=0; i<func->param.size(); i++ )
	{
		// TODO: Generate an expression with the correct type...
		param_value.push_back( new Expression( *target ) );
	}
}

void OutputStandardFuncName( eBinaryOps eFunc, std::ostream &out )
{
	switch ( eFunc )
	{
	// Math Ops
	case eAdd:		out << "+";		break;
	case eSub:		out << "-";		break;
	case eMul:		out << "*";		break;
	case eDiv:		out << "/";		break;
	case eMod:		out << "%";		break;

	// Logical Ops
	case eAnd:		out << "&&";	break;
	case eOr:		out << "||";	break;
	case eCmpEq:	out << "==";	break;
	case eCmpNe:	out << "!=";	break;
	case eCmpGt:	out << ">";		break;
	case eCmpLt:	out << "<";		break;
	case eCmpLe:	out << "<=";	break;
	case eCmpGe:	out << ">=";	break;

	// Bitwise Ops
	case eBitAnd:	out << "&";		break;
	case eBitOr:	out << "|";		break;
	case eBitXor:	out << "^";		break;
	case eLShift:	out << "<<";	break;
	case eRShift:	out << ">>";	break;
	}
}

void OutputActualParamExpression( Expression *expr, std::ostream *pOut )
{
	std::ostream &out = *pOut;
	if ( needcomma.back() ) out << ", ";
	needcomma.back() = true;
	expr->Output( out );
}

void OutputExpressionVector( vector<Expression*> &var, std::ostream &out )
{
	needcomma.push_back( false );
	for_each( var.begin(), var.end(), std::bind2nd(std::ptr_fun(OutputActualParamExpression), &out) );
	needcomma.pop_back();
}

void FunctionInvocation::Output( std::ostream &out )
{
	if ( isStandardFunc )
	{
		// One of the Binary Ops
		out << "( ";
		param_value[0]->Output( out );
		out << " ";
		OutputStandardFuncName( eFunc, out );
		out << " ";
		param_value[1]->Output( out );
	}
	else
	{
		out << func->name << "( ";
		OutputExpressionVector( param_value, out );
	}
	out << " )";
};


