//
// Random Program Generator
// Bryan Turner (bryan.turner@pobox.com)
// July, 2005
//

#include "Common.h"

#include "Type.h"

// ---------------------------------------------------------------------
// List of all types used in the program
vector<Type*> AllTypes;

// --------------------------------------------------------------
string GenerateRandomCharConstant()
{
	string ch;
	ch = string("0x") + RandomHexDigits(2);
	return ch;
}

// --------------------------------------------------------------
string GenerateRandomIntConstant()
{
	// Int constant - Max 8 Hex digits on 32-bit platforms
	string val = "0x" + RandomHexDigits( 8 );
	return val;
}

// --------------------------------------------------------------
string GenerateRandomShortConstant()
{
	// Short constant - Max 4 Hex digits on 32-bit platforms
	string val = "0x" + RandomHexDigits( 4 );
	return val;
}

// --------------------------------------------------------------
string GenerateRandomLongConstant()
{
	// Long constant - Max 8 Hex digits on 32-bit platforms
	string val = "0x" + RandomHexDigits( 8 );
	return val;
}

// --------------------------------------------------------------
string GenerateRandomFloatConstant()
{
	// Generate a random floating point value with up to 10 digits of precision. (should look up precision of float/double.. 23 bits for IEEE-32?)
	string val = RandomDigits(5) + "." + RandomDigits(5);
	return val;
}

// --------------------------------------------------------------
string GenerateRandomSimpleConstant( eSimpleType eType )
{
	switch ( eType )
	{
	case eChar:     return GenerateRandomCharConstant();
	case eInt:      return GenerateRandomIntConstant();
	case eShort:    return GenerateRandomShortConstant();
	case eLong:     return GenerateRandomLongConstant();
	case eUChar:    return GenerateRandomCharConstant();
	case eUInt:     return GenerateRandomIntConstant();
	case eUShort:   return GenerateRandomShortConstant();
	case eULong:    return GenerateRandomLongConstant();
//	case eLongLong: return GenerateRandomLongLongConstant();
//	case eFloat:    return GenerateRandomFloatConstant();
//	case eDouble:   return GenerateRandomFloatConstant();
	}

	return string("ERROR /* Const Generator Error */");
}

bool AllTypesProbability()
{
	if ( AllTypes.size() < 10 ) return true;	// Always have at least 10 types in the program.
	return (genrand() % 1000) > 500;		// 50% probability for each additional type.
}

// ---------------------------------------------------------------------
Type *GenerateRandomType()
{
	Type *t = new Type();
	t->eType = eSimple; //(eTypeDesc) (genrand() % MAX_TYPE_DESC);		// TODO: Test other type creation..
	switch ( t->eType )
	{
	default:
	case eSimple:	t->simple_type = (eSimpleType) (1+(genrand() % (MAX_SIMPLE_TYPES-1)));	// No Void..
//	case ePointer:
//	case eArray:
//	case eUnion:
//	case eStruct:
	}
	return t;
}

// ---------------------------------------------------------------------
void GenerateAllTypes()
{
	while ( AllTypesProbability() )
	{
		AllTypes.push_back( GenerateRandomType() );
	}
}

// ---------------------------------------------------------------------
Type *SelectRandomType()
{
	return AllTypes[ genrand() % AllTypes.size() ];
}

unsigned long Type::SizeInBytes() const
{
	switch ( eType )
	{
	default:
	case eSimple:
		switch (simple_type)
		{
		case eChar:		return 1;
		case eInt:		return 4;
		case eShort:	return 2;
		case eLong:		return 4;
		case eUChar:	return 1;
		case eUInt:		return 4;
		case eUShort:	return 2;
		case eULong:	return 4;
//		case eLongLong:	return 8;
//		case eFloat:	return 4;
//		case eDouble:	return 8;
		}
		break;
//	case ePointer:	return 4;
//	case eArray:
//	case eUnion:
//	case eStruct:
	}
	return 0;
}

// ---------------------------------------------------------------------
void Type::Output( std::ostream &out ) const
{
	switch ( eType )
	{
	case eSimple:
		switch ( simple_type )
		{
		case eVoid:		out << "void"; break;
		case eChar:		out << "char"; break;
		case eInt:		out << "int"; break;
		case eShort:	out << "short"; break;
		case eLong:		out << "long"; break;
		case eUChar:	out << "unsigned char"; break;
		case eUInt:		out << "unsigned int"; break;
		case eUShort:	out << "unsigned short"; break;
		case eULong:	out << "unsigned long"; break;
//		case eFloat:	out << "float"; break;
//		case eDouble:	out << "double"; break;
//      case eLongLong: out << "long long"; break;
		}
		break;
//	case ePointer:  ptr_type->OutputType( out ); out << "*"; break;
//	case eUnion:
//	case eStruct:
//	case eArray:	ptr_type->OutputType( out> ); out << "[]"; break;	// TODO: Output correct array subscripts..
	}
};

